@@ -726,7 +726,7 @@ int execute_cgi(t_session *session) {
 								free(cache_buffer);
 								cache_buffer = NULL;
 							} else {
-								memcpy(cache_buffer	+ cache_size, cgi_info.input_buffer, cgi_info.input_len);
+								memcpy(cache_buffer + cache_size, cgi_info.input_buffer, cgi_info.input_len);
 								cache_size += cgi_info.input_len;
 								*(cache_buffer + cache_size) = '\0';
 							}
@@ -883,7 +883,7 @@ int execute_cgi(t_session *session) {
 			case cgi_END_OF_DATA:
 				if (in_body) {
 					retval = rs_QUIT;
-					if (send_in_chunks) {
+					if (send_in_chunks && (session->request_method != HEAD)) {
 						if (send_chunk(session, NULL, 0) == -1) {
 							retval = rs_DISCONNECT;
 						}
@@ -1329,14 +1329,32 @@ int handle_xml_file(t_session *session) {
 #endif
 
 #ifdef ENABLE_RPROXY
+static int find_chunk_size(char *buffer) {
+	int chunk_extra_size, chunk_size;
+
+	if (strstr(buffer, "\r\n") == NULL) {
+		return -1;
+	}
+
+	chunk_extra_size = 4;
+	chunk_size = 0;
+	while (*buffer != '\r') {
+		chunk_size = (16 * chunk_size) + (int)hex_to_int(*buffer);
+		chunk_extra_size++;
+		buffer++;
+	}
+
+	return chunk_extra_size + chunk_size;
+}
+
 int proxy_request(t_session *session, t_rproxy *rproxy) {
 	t_rproxy_options options;
 	t_rproxy_webserver webserver;
 	t_rproxy_result rproxy_result;
 	char buffer[RPROXY_BUFFER_SIZE + 1], *end_of_header, *str, *eol;
-	int bytes_read, bytes_in_buffer = 0, result = 200, code, poll_result;
-	int can_read, content_length = -1, content_read = 0;
-	bool header_read = false, keep_reading = true, keep_alive, chunked = false;
+	int bytes_read, bytes_in_buffer = 0, result = 200, code, poll_result, send_size;
+	int can_read, content_length = -1, content_read = 0, chunk_size, chunk_total, header_size;
+	bool header_read = false, keep_reading = true, keep_alive, chunked_transfer = false;
 	struct pollfd poll_data;
 	time_t deadline;
 #ifdef ENABLE_CACHE
@@ -1356,12 +1374,14 @@ int proxy_request(t_session *session, t_rproxy *rproxy) {
 	 */
 	if (session->request_method == GET) {
 		if ((cached_object = search_cache_for_rproxy_output(session)) != NULL) {
-			if (send_header(session) == -1) {
-				result = rs_DISCONNECT;
-			} else if (send_buffer(session, cached_object->data, cached_object->size) == -1) {
+			if (send_buffer(session, cached_object->data, cached_object->size) == -1) {
 				result = rs_DISCONNECT;
 			}
 
+			if (cached_object->close_connection) {
+				session->keep_alive = false;
+			}
+
 			done_with_cached_object(cached_object, false);
 
 			return result;
@@ -1393,14 +1413,16 @@ int proxy_request(t_session *session, t_rproxy *rproxy) {
 	 */
 	if (session->rproxy_kept_alive) {
 		if (recv(session->rproxy_socket, buffer, 1, MSG_DONTWAIT | MSG_PEEK) == -1) {
+			if (errno != EAGAIN) {
 #ifdef ENABLE_SSL
-			if (session->rproxy_use_ssl) {
-				ssl_close(&(session->rproxy_ssl));
-			}
+				if (session->rproxy_use_ssl) {
+					ssl_close(&(session->rproxy_ssl));
+				}
 #endif
-			close(session->rproxy_socket);
+				close(session->rproxy_socket);
 
-			session->rproxy_kept_alive = false;
+				session->rproxy_kept_alive = false;
+			}
 		}
 	}
 
@@ -1480,12 +1502,16 @@ int proxy_request(t_session *session, t_rproxy *rproxy) {
 					}
 				}
 
+				if (RPROXY_BUFFER_SIZE - bytes_in_buffer > 0) {
 #ifdef ENABLE_SSL
-			    if (webserver.use_ssl) {
-					bytes_read = ssl_receive(&(webserver.ssl), buffer + bytes_in_buffer, RPROXY_BUFFER_SIZE - bytes_in_buffer);
-				} else
+					if (webserver.use_ssl) {
+						bytes_read = ssl_receive(&(webserver.ssl), buffer + bytes_in_buffer, RPROXY_BUFFER_SIZE - bytes_in_buffer);
+					} else
 #endif
-					bytes_read = read(webserver.socket, buffer + bytes_in_buffer, RPROXY_BUFFER_SIZE - bytes_in_buffer);
+						bytes_read = read(webserver.socket, buffer + bytes_in_buffer, RPROXY_BUFFER_SIZE - bytes_in_buffer);
+				} else {
+					bytes_read = -1;
+				}
 
 				switch (bytes_read) {
 					case -1:
@@ -1501,10 +1527,12 @@ int proxy_request(t_session *session, t_rproxy *rproxy) {
 					default:
 						/* Read first line and extract return code
 						 */
-						if (header_read == false) {
-							bytes_in_buffer += bytes_read;
+						bytes_in_buffer += bytes_read;
+						*(buffer + bytes_in_buffer) = '\0';
 
-							*(buffer + bytes_in_buffer) = '\0';
+						if (header_read == false) {
+							/* Look for header
+							 */
 							if ((end_of_header = strstr(buffer, "\r\n\r\n")) != NULL) {
 								*(end_of_header + 2) = '\0';
 								if ((code = extract_http_code(buffer)) != -1) {
@@ -1520,7 +1548,7 @@ int proxy_request(t_session *session, t_rproxy *rproxy) {
 								}
 
 #ifdef ENABLE_CACHE
-								if (code == 200) {
+								if ((code == 200) && (session->request_method == GET)) {
 									if ((cache_time = rproxy_cache_time(session, buffer)) > 0) {
 										if ((cache_buffer = (char*)malloc(session->config->cache_max_filesize + 1)) != NULL) {
 											*(cache_buffer + session->config->cache_max_filesize) = '\0';
@@ -1533,46 +1561,47 @@ int proxy_request(t_session *session, t_rproxy *rproxy) {
 
 								/* Parse content length
 								 */
-								if ((str = strstr(buffer, hs_conlen)) != NULL) {
+								if (session->request_method == HEAD) {
+									content_length = 0;
+								} else if (((code >= 100) && (code < 200)) || (code == 204) ||
+								            (code == 302) || (code == 303) || (code == 304)) {
+									content_length = 0;
+								} else if ((str = strcasestr(buffer, hs_conlen)) != NULL) {
 									str += strlen(hs_conlen);
 									if ((eol = strchr(str, '\r')) != NULL) {
 										*eol = '\0';
 										content_length = str2int(str);
 										*eol = '\r';
 									}
+								} else {
+									content_length = 0;
 								}
 
-								/* Determine if is chunked encoding
+								/* Determine if is chunked transfer encoding
 								 */
-								if (strstr(buffer, hs_chunked) != NULL) {
-									chunked = true;
+								if (strcasestr(buffer, hs_chunked) != NULL) {
+									chunked_transfer = true;
+									content_length = -1;
+									header_size = end_of_header + 4 - buffer;
 								}
 
 								/* Determine connection type
 								 */
-								if (strstr(buffer, " HTTP/1.0\r\n") != NULL) {
+								if (strncmp(buffer, "HTTP/1.0 ", 9) == 0) {
 									keep_alive = false;
-								} else if (strstr(buffer, " HTTP/1.1\r\n") != NULL) {
-									keep_alive = true;
-									if ((str = strstr(buffer, hs_conn)) != NULL) {
+								} else if (strncmp(buffer, "HTTP/1.1 ", 9) == 0) {
+									if ((str = strcasestr(buffer, hs_conn)) != NULL) {
 										str += strlen(hs_conn);
-										if ((eol = strchr(str, '\r')) != NULL) {
-											*eol = '\0';
-											if (strcmp(str, "close") == 0) {
-												keep_alive = false;
-											}
-											*eol = '\r';
+										if (strncmp(str, "close\r\n", 7) == 0) {
+											keep_alive = false;
 										}
 									}
 								}
 
 								content_read = bytes_in_buffer - (end_of_header + 4 - buffer);
+								header_read = true;
 
 								*(end_of_header + 2)= '\r';
-
-								bytes_read = bytes_in_buffer;
-								bytes_in_buffer = 0;
-								header_read = true;
 							} else if (bytes_in_buffer == RPROXY_BUFFER_SIZE) {
 								result = -1;
 								keep_reading = false;
@@ -1582,41 +1611,79 @@ int proxy_request(t_session *session, t_rproxy *rproxy) {
 								continue;
 							}
 						} else {
+							/* Dealing with body
+							 */
 							content_read += bytes_read;
-							*(buffer + bytes_read) = '\0';
 						}
 
 						if (content_read == content_length) {
 							keep_reading = false;
 						}
 
-						if (chunked && (bytes_read >= 7)) {
-							if (memcmp(buffer + bytes_read - 7, "\r\n0\r\n\r\n", 7) == 0) {
-								keep_reading = false;
+						/* Determine what to send
+						 */
+						if (chunked_transfer) {
+							/* Send chunk
+							 */
+							chunk_total = header_size;
+							while ((chunk_size = find_chunk_size(buffer + chunk_total)) != -1) {
+								if (chunk_total + chunk_size <= bytes_in_buffer) {
+									chunk_total += chunk_size;
+
+									if (chunk_size == 5) {
+										keep_reading = false;
+									}
+								} else {
+									break;
+								}
 							}
+							
+							if (bytes_in_buffer >= chunk_total) {
+								send_size = chunk_total;
+							} else {
+								send_size = 0;
+							}
+						} else {
+							/* Send complete buffer
+							 */
+							send_size = bytes_in_buffer;
 						}
 
-						/* Send data to client
+						/* Send buffer content
 						 */
-						if (send_buffer(session, buffer, bytes_read) == -1) {
-							result = -1;
-							keep_reading = false;
-							keep_alive = false;
-							break;
-						}
+						if (send_size > 0) {
+							if (send_buffer(session, buffer, send_size) == -1) {
+								result = -1;
+								keep_reading = false;
+								keep_alive = false;
+								break;
+							}
 
 #ifdef ENABLE_CACHE
-						if (cache_buffer != NULL) {
-							if ((off_t)(cache_size + bytes_read) > session->config->cache_max_filesize) {
-								clear_free(cache_buffer, cache_size);
-								cache_buffer = NULL;
+							/* Add output to cache buffer
+							 */
+							if (cache_buffer != NULL) {
+								if ((off_t)(cache_size + send_size) > session->config->cache_max_filesize) {
+									clear_free(cache_buffer, cache_size);
+									cache_buffer = NULL;
+								} else {
+									memcpy(cache_buffer + cache_size, buffer, send_size);
+									cache_size += send_size;
+									*(cache_buffer + cache_size) = '\0';
+								}
+							}
+#endif
+
+							if (chunked_transfer) {
+								bytes_in_buffer -= chunk_total;
+								memmove(buffer, buffer + chunk_total, bytes_in_buffer);
+								*(buffer + bytes_in_buffer) = '\0';
+
+								header_size = 0;
 							} else {
-								memcpy(cache_buffer + cache_size, buffer, bytes_read);
-								cache_size += bytes_read;
-								*(cache_buffer + cache_size) = '\0';
+								bytes_in_buffer = 0;
 							}
 						}
-#endif
 
 						session->data_sent = true;
 				}
@@ -1625,7 +1692,7 @@ int proxy_request(t_session *session, t_rproxy *rproxy) {
 
 #ifdef ENABLE_CACHE
 	if (cache_buffer != NULL) {
-		add_rproxy_output_to_cache(session, cache_buffer, cache_size, cache_time);
+		add_rproxy_output_to_cache(session, cache_buffer, cache_size, cache_time, keep_alive);
 		clear_free(cache_buffer, cache_size);
 	}
 #endif
