@@ -382,6 +382,7 @@ static int serve_client(t_session *session) {
 #else
 		remove_port_from_hostname(session->hostname);
 #endif
+		reset_http_header_strlen("Host:", session->http_headers);
 
 		if ((host_record = get_hostrecord(session->config->first_host, session->hostname, session->binding)) != NULL) {
 			session->host = host_record;
@@ -1181,7 +1182,7 @@ static int add_thread_to_pool(t_session *session) {
 		free(new_thread);
 		return -1;
 	} else if (pthread_create(&(new_thread->worker), &child_attr, (void*)thread_wait_loop, (void*)new_thread) != 0) {
-		fprintf(stderr, "pthread create error.\n");
+		perror("pthread create error");
 		pthread_attr_destroy(&child_attr);
 		free(new_thread);
 		return -1;
